# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/charts.ipynb (unless otherwise specified).

__all__ = ['alt', 'Chart', 'confusion_matrix', 'CM']

# Cell
# export
import altair as alt
from altair import Chart
alt = alt
Chart = Chart
from .loader import *

# Cell
def confusion_matrix(df=None, truth=None, pred=None, mapping=None):
    if df is None:
        df = pd.DataFrame({'truth': truth, 'pred': pred})
        truth = 'truth'
        pred = 'pred'
    threshold = len(df)
    if mapping:
        assert isinstance(mapping, dict), 'mapping should be a dictionary'
        df[truth] = df[truth].map(lambda x: mapping[x])
        df[pred] = df[pred].map(lambda x: mapping[x])

    sz = 450 if len(df[truth].unique()) > 4 else 250
    base = Chart(df, height=sz, width=sz).transform_aggregate(
        num_vals='count()',
        groupby=[truth, pred]
    ).transform_calculate(
        rev_num_vals='-(datum.num_vals) + max(datum.num_vals)',
    ).encode(
        alt.Y(f'{truth}:O', scale=alt.Scale(paddingInner=0)),
        alt.X(f'{pred}:O', scale=alt.Scale(paddingInner=0)),
    )

    hm = base.mark_rect().encode(
        color=alt.Color(
            'num_vals:Q', scale=alt.Scale(scheme="lightorange"),
            legend=None
        )
    )

    tx = base.mark_text(baseline='middle').encode(
        text='num_vals:Q',
#         color=alt.Color(alt.value('gray'))
#         color='rev_num_vals:Q'
#         color=alt.Color(
#             'num_vals:Q', scale=alt.Scale(scheme="redyellowgreen"),
#         )
        color=alt.condition(
            alt.datum.num_vals > threshold,
            alt.value('black'),
            alt.value('black'))
    )

    try:
        from sklearn.metrics import classification_report
        print(classification_report(df[truth], df[pred]))
    except:
        logger.info('Skipping Report')
    return hm + tx

CM = confusion_matrix


# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/markups.ipynb (unless otherwise specified).

__all__ = ['isnamedtupleinstance', 'unpack', 'read_json', 'write_json', 'AttrDict', 'pretty_json', 'read_yaml',
           'write_yaml', 'Config']

# Cell
def isnamedtupleinstance(x):
    _type = type(x)
    bases = _type.__bases__
    if len(bases) != 1 or bases[0] != tuple:
        return False
    fields = getattr(_type, '_fields', None)
    if not isinstance(fields, tuple):
        return False
    return all(type(i)==str for i in fields)

def unpack(obj):
    if isinstance(obj, dict):
        return {key: unpack(value) for key, value in obj.items()}
    elif isinstance(obj, list):
        return [unpack(value) for value in obj]
    elif isnamedtupleinstance(obj):
        return {key: unpack(value) for key, value in obj._asdict().items()}
    elif isinstance(obj, tuple):
        return tuple(unpack(value) for value in obj)
    else:
        return obj

# Cell
import json
from .paths import *
from .logger import *

def read_json(fpath):
    import json
    with open(fpath, 'r') as f:
        return json.load(f)

def write_json(obj, fpath):
    def set_default(obj):
        if isinstance(obj, set):
            return list(obj)
        raise TypeError

    logger.opt(depth=1).log("DEBUG", f"Dumping json to {fpath}")
    with open(fpath, "w") as f:
        json.dump(obj, f, indent=4, default=set_default)
    return P(fpath)

class AttrDict(object):
    def __init__(self, data):
        for name, value in data.items():
            setattr(self, name, self._wrap(value))

    def items(self):
        return self.__dict__.items()

    def keys(self):
        return self.__dict__.keys()

    def values(self):
        return self.__dict__.values()

    def _wrap(self, value):
        if isinstance(value, (tuple, list, set, frozenset)):
            return type(value)([self._wrap(v) for v in value])
        else:
            return AttrDict(value) if isinstance(value, dict) else value

    __getitem__ = lambda self, x: getattr(self, x)
    __setitem__ = lambda self, k, v: setattr(self, k, self._wrap(v))

    def __repr__(self):
        return '{%s}' % str(', '.join("'%s': %s" % (k, repr(v)) for (k, v) in self.__dict__.items()))

    def __dir__(self):
        return self.__dict__.keys()

    def __contains__(self, key):
        return key in self.__dict__.keys()

    def __delitem__(self, key):
        del self.__dict__[key]

    def to_dict(self):
        d = {}
        for k in dir(self):
            v = self[k]
            if isinstance(v, AttrDict):
                v = v.to_dict()
            if isinstance(v, (tuple, list, set, frozenset)):
                v = [_v.to_dict() if isinstance(_v, AttrDict) else _v for _v in v]
            d[k] = v
        return d

    def pretty(self, print_with_logger=False, *args, **kwargs):
        pretty_json(self.to_dict(), print_with_logger=print_with_logger, *args, **kwargs)

    def __eq__(self, other):
        return AttrDict(other).to_dict() == self.to_dict()


def pretty_json(i, fpath=None, indent=4, print_with_logger=True):
    def set_default(obj):
        if isinstance(obj, set):
            return list(obj)
    assert isinstance(i, (dict, list))
    i = unpack(i)
    dump = json.dumps(i, indent=indent, default=set_default)
    if fpath:
        makedir(parent(fpath))
        logger.info(f'Dumped a pretty file to {fpath}')
        with open(fpath, 'w') as f:
            json.dump(i, f, indent=indent, default=set_default)
            return
    if print_with_logger:
        return logger.opt(depth=1).log('DEBUG', f'\n{dump}')
    else:
        print(dump)

# Cell
import yaml

def read_yaml(file):
    with open(file, 'r') as stream:
        try:
            return yaml.safe_load(stream)
        except yaml.YAMLError as exc:
            print(exc)

def write_yaml(content, fpath):
    with open(fpath, 'w') as outfile:
        yaml.dump(content, outfile, default_flow_style=False)

# Cell
from .thinc_parser.parser import Config
Config = Config


# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"alt": "charts.ipynb",
         "Chart": "charts.ipynb",
         "confusion_matrix": "charts.ipynb",
         "CM": "charts.ipynb",
         "get_console": "logging.ipynb",
         "console": "logging.ipynb",
         "render": "logging.ipynb",
         "logger": "logging.ipynb",
         "Debug": "logging.ipynb",
         "Info": "logging.ipynb",
         "Warn": "logging.ipynb",
         "Excep": "logging.ipynb",
         "reset_logger_width": "logging.ipynb",
         "isnamedtupleinstance": "markups.ipynb",
         "unpack": "markups.ipynb",
         "read_json": "markups.ipynb",
         "write_json": "markups.ipynb",
         "AttrDict": "registry.ipynb",
         "pretty_json": "markups.ipynb",
         "read_yaml": "markups.ipynb",
         "write_yaml": "markups.ipynb",
         "Config": "registry.ipynb",
         "input_to_str": "paths.ipynb",
         "output_to_path": "paths.ipynb",
         "size": "paths.ipynb",
         "extn": "paths.ipynb",
         "Glob": "paths.ipynb",
         "sample": "paths.ipynb",
         "mv": "paths.ipynb",
         "cp": "paths.ipynb",
         "rm": "paths.ipynb",
         "P": "paths.ipynb",
         "P.ls": "paths.ipynb",
         "P.__repr__": "paths.ipynb",
         "isdir": "paths.ipynb",
         "makedir": "paths.ipynb",
         "fname": "paths.ipynb",
         "fname2": "paths.ipynb",
         "stem": "paths.ipynb",
         "stems": "paths.ipynb",
         "parent": "paths.ipynb",
         "find": "paths.ipynb",
         "zip_files": "paths.ipynb",
         "unzip_file": "paths.ipynb",
         "md5": "paths.ipynb",
         "remove_duplicates": "paths.ipynb",
         "readlines": "paths.ipynb",
         "read_lines": "paths.ipynb",
         "writelines": "paths.ipynb",
         "write_lines": "paths.ipynb",
         "rename_batch": "paths.ipynb",
         "dumpdill": "paths.ipynb",
         "loaddill": "paths.ipynb",
         "dill": "paths.ipynb",
         "parse": "registry.ipynb",
         "parse_and_resolve": "registry.ipynb",
         "registry": "registry.ipynb",
         "train_test_split": "sklegos.ipynb",
         "ColumnSelector": "sklegos.ipynb",
         "transform": "sklegos.ipynb",
         "GroupedEstimator": "sklegos.ipynb",
         "GroupedPredictor": "sklegos.ipynb",
         "EstimatorTransformer": "sklegos.ipynb",
         "MakeFrame": "sklegos.ipynb",
         "ImputeMissingValues": "sklegos.ipynb",
         "LambdaTransformer": "sklegos.ipynb",
         "Cat2Num": "sklegos.ipynb",
         "SplitDateColumn": "sklegos.ipynb"}

modules = ["charts.py",
           "logger.py",
           "markup.py",
           "paths.py",
           "registry.py",
           "sklegos.py"]

doc_url = "https://sizhky.github.io/torch_snippets/"

git_url = "https://github.com/sizhky/torch_snippets/tree/master/"

def custom_doc_links(name): return None



# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/registry.ipynb (unless otherwise specified).

__all__ = ['parse', 'parse_and_resolve', 'Config', 'AttrDict', 'registry', 'parse', 'parse_and_resolve']

# Cell
from .markup import Config, AttrDict
from .thinc_parser.parser import registry
Config = Config
AttrDict = AttrDict
registry = registry

def parse(file):
    return AttrDict(Config().from_disk(file))

def parse_and_resolve(file):
    return AttrDict(registry.resolve(Config().from_disk(file)))

# Cell
def parse(file):
    return AttrDict(Config().from_disk(file))

def parse_and_resolve(file):
    return AttrDict(registry.resolve(Config().from_disk(file)))